//
//  opcoderPlugIn.m
//  opcoder
//
//  Created by Dustin O'Connor on 5/12/11.
//  Copyright 2011 umaine. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "opcoderPlugIn.h"

#define	kQCPlugIn_Name				@"opcoder"
#define	kQCPlugIn_Description		@"opcoder description"

@implementation opcoderPlugIn
@dynamic inputVoice, inputText, outputOpcode, inputRun,inputRate, outputWords, inputVol;
@synthesize code, words;
/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

+ (NSDictionary *)attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary *)attributesForPropertyPortWithKey:(NSString *)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
	return nil;
}

+ (QCPlugInExecutionMode)executionMode
{
	/*
	Return the execution mode of the plug-in: kQCPlugInExecutionModeProvider, kQCPlugInExecutionModeProcessor, or kQCPlugInExecutionModeConsumer.
	*/
	
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode)timeMode
{
	/*
	Return the time dependency mode of the plug-in: kQCPlugInTimeModeNone, kQCPlugInTimeModeIdle or kQCPlugInTimeModeTimeBase.
	*/
	
	return kQCPlugInTimeModeIdle;
}

- (id)init
{
	self = [super init];
	if (self) {
        
        self.words = @"";
		/*
		Allocate any permanent resource required by the plug-in.
		*/
	}
	
	return self;
}

- (void)finalize
{
	/*
	Release any non garbage collected resources created in -init.
	*/
	
	[super finalize];
}

- (void)dealloc
{
	/*
	Release any resources created in -init.
	*/
    [words release];

    
	[super dealloc];
}


@end

@implementation opcoderPlugIn (Execution)

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakPhoneme:(short)phonemeOpcode
{   
    self.code = (double)phonemeOpcode;
    
}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender didFinishSpeaking:(BOOL)success
{

}

- (void)speechSynthesizer:(NSSpeechSynthesizer *)sender willSpeakWord:(NSRange)wordToSpeak ofString:(NSString *)text
{
    
    self.words = [text substringWithRange:wordToSpeak];
    NSLog(@"range finder %@   ", [text substringWithRange:wordToSpeak]);
   
   

}

- (BOOL)startExecution:(id <QCPlugInContext>)context
{
	_synthetizer = [[NSSpeechSynthesizer alloc] initWithVoice:[NSSpeechSynthesizer defaultVoice]];
	
	return (_synthetizer ? YES : NO);
	return YES;
}

- (void)enableExecution:(id <QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance starts being used by Quartz Composer.
	*/
}

- (BOOL)execute:(id <QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary *)arguments
{
	BOOL						speakText = [self didValueForInputKeyChange:@"inputText"];
   
    [_synthetizer setRate:(float)self.inputRate];
    [_synthetizer setVolume:(float) self.inputVol];  

     
     self.outputOpcode = self.code;
    
    self.outputWords = self.words;
    
   

      if(self.inputRun)
          {
        
    /* Update synthetizer voice if necessary */
	if([self didValueForInputKeyChange:@"inputVoice"] && [self.inputVoice length] ) {
		[_synthetizer stopSpeaking];
		[_synthetizer setVoice:[@"com.apple.speech.synthesis.voice." stringByAppendingString:self.inputVoice]];
       	speakText = YES;
	}
    
       [_synthetizer stopSpeaking];
       [_synthetizer setVoice:[@"com.apple.speech.synthesis.voice." stringByAppendingString:self.inputVoice]];
      

      speakText = YES;
	
	/* Speak text if necessary */
	if(speakText) {
        [_synthetizer stopSpeaking];
		if([self.inputText length])
        [_synthetizer setUsesFeedbackWindow:NO];
        [_synthetizer setDelegate:self];
        
		[_synthetizer startSpeakingString:self.inputText];
 
	}
      }
	
	return YES;
}

- (void)disableExecution:(id <QCPlugInContext>)context
{
	/*
	Called by Quartz Composer when the plug-in instance stops being used by Quartz Composer.
	*/
}

- (void)stopExecution:(id <QCPlugInContext>)context
{
    [_synthetizer stopSpeaking];
	[_synthetizer release];
	_synthetizer = nil;
}

@end
